import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.WindowInsets
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.systemBars
import androidx.compose.foundation.layout.windowInsetsPadding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.rememberLazyListState
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.foundation.text.selection.SelectionContainer
import androidx.compose.material.Icon
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableFloatStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.alpha
import androidx.compose.ui.draw.shadow
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.input.nestedscroll.NestedScrollConnection
import androidx.compose.ui.input.nestedscroll.NestedScrollSource
import androidx.compose.ui.input.nestedscroll.nestedScroll
import androidx.compose.ui.unit.Velocity
import androidx.compose.ui.unit.dp
import details.ingredients
import details.ingredientsHeader
import details.recipeHeader
import details.steps
import details.stepsHeader
import model.Recipe
import sensor.Listener
import sensor.SensorData
import sensor.SensorManager
import view.RecipeScaleSlider
import kotlin.math.PI


@OptIn(ExperimentalFoundationApi::class)
@Composable
fun RecipeDetailsSmall(
    recipe: Recipe,
    goBack: () -> Unit,
    sensorManager: SensorManager?,
    isWebApp: Boolean,
) {
    val imageRotation = remember { mutableStateOf(0) }
    val sensorDataLive = remember { mutableStateOf(SensorData(0.0f, 0.0f)) }

    sensorManager?.registerListener(object : Listener {
        override fun onUpdate(sensorData: SensorData) {
            sensorDataLive.value = sensorData
        }
    })

    val toolbarOffsetHeightPx = remember { mutableStateOf(340f) }
    val nestedScrollConnection = remember {
        object : NestedScrollConnection {
            override fun onPreScroll(
                available: Offset, source: NestedScrollSource
            ): Offset {
                val delta = available.y
                val newOffset = toolbarOffsetHeightPx.value + delta
                toolbarOffsetHeightPx.value = newOffset.coerceIn(0f, 340f)
                imageRotation.value += (available.y * 0.5).toInt()
                return Offset.Zero
            }

            override fun onPostScroll(
                consumed: Offset, available: Offset, source: NestedScrollSource
            ): Offset {
                val delta = available.y
                return super.onPostScroll(consumed, available, source)
            }

            override suspend fun onPreFling(available: Velocity): Velocity {
                imageRotation.value += available.y.toInt()
                return super.onPreFling(available)
            }
        }
    }

    val candidateHeight = maxOf(toolbarOffsetHeightPx.value, 300f)
    val listState = rememberLazyListState()
    val (fraction, setFraction) = remember { mutableStateOf(1f) }
    var slider by remember { mutableFloatStateOf(1f) }
    var reduce by remember { mutableStateOf(true) }

        Box(
            modifier = Modifier.fillMaxSize()
                .background(color = if (recipe.bgColor == MaterialTheme.colorScheme.primaryContainer) MaterialTheme.colorScheme.secondaryContainer else MaterialTheme.colorScheme.background)
                .padding(bottom = 12.dp)
        ) {
            Column {
                LazyColumn(
                    modifier = Modifier.fillMaxSize().nestedScroll(nestedScrollConnection),
                    state = listState
                ) {

                    stickyHeader {
                        Row {
                            if (isWebApp)
                                BackButton(goBack)

                            Box(
                                modifier = Modifier.fillMaxWidth()
                                    .background(color = if (recipe.bgColor == MaterialTheme.colorScheme.primaryContainer) MaterialTheme.colorScheme.secondaryContainer else MaterialTheme.colorScheme.background)
                            ) {
                                SelectionContainer {
                                    Text(
                                        text = recipe.title.plus(" ").plus(slider).plus("x"),
                                        style = MaterialTheme.typography.headlineLarge
                                    )
                                }
                            }
                        }
                    }

                    item {
                        recipeHeader(recipe, false, slider)
                    }

                    item {
                        RecipeScaleSlider(
                            sliderPosition = slider,
                            steps = if(reduce) 2 else 5,
                            reduce = reduce,
                            onClick = {
                                reduce = !reduce
                                slider = 1f
                            },
                            rangeEnd = if(reduce) 1f else 4f,
                            rangeStart = if(reduce) .25f else 1f,) { slider = it}
                    }



                    item {
                        ingredientsHeader(recipe)
                    }

                    ingredients(recipe, slider)

                    item {
                        stepsHeader(recipe)
                    }

                    steps(recipe, slider)
                }
            }

            Box(modifier = Modifier.windowInsetsPadding(WindowInsets.systemBars).size(50.dp)
                .padding(10.dp).alpha(
                    alpha = if (fraction <= 0) 1f else 0f,
                ).background(
                    color = Color.Black, shape = RoundedCornerShape(50)
                ).shadow(elevation = 16.dp).padding(5.dp).clickable {
                    goBack()
                }) {
                Icon(
                    imageVector = Icons.AutoMirrored.Default.ArrowBack,
                    contentDescription = null,
                    tint = recipe.bgColor,
                    modifier = Modifier.size(30.dp)
                )
            }
        }
}
